@@ -2,7 +2,8 @@
  * AT-SPI - Assistive Technology Service Provider Interface
  * (Gnome Accessibility Project; http://developer.gnome.org/projects/gap)
  *
- * Copyright 2001 Sun Microsystems Inc., Ximian Inc.
+ * Copyright 2001, 2002 Sun Microsystems Inc.,
+ * Copyright 2001, 2002 Ximian, Inc., Ximian Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -26,6 +27,9 @@
 #include <stdio.h>
 #include <libbonobo.h>
 #include "desktop.h"
+#include <atk/atkcomponent.h>
+#include <gdk/gdkscreen.h>
+#include <gdk/gdkx.h>
 
 /* SpiDesktop signals */
 enum {
@@ -44,18 +48,109 @@ typedef struct {
 	Accessibility_Application ref;
 } Application;
 
+static gboolean exiting = FALSE;
+
 /* A pointer to our parent object class */
 static SpiAccessibleClass *parent_class;
 
+#define SPI_TYPE_ATK_DESKTOP		(spi_atk_desktop_get_type ())
+#define SPI_ATK_DESKTOP(o)		(G_TYPE_CHECK_INSTANCE_CAST ((o), SPI_TYPE_ATK_DESKTOP, SpiAtkDesktop))
+#define SPI_IS_ATK_DESKTOP(o)		(G_TYPE_CHECK_INSTANCE_TYPE ((o), SPI_TYPE_ATK_DESKTOP))
+
+typedef struct {
+	AtkObject parent;
+
+	GdkScreen *screen;
+} SpiAtkDesktop;
+
+typedef struct {
+	AtkObjectClass parent;
+} SpiAtkDesktopClass;
+
+static void spi_atk_desktop_init (SpiAtkDesktop *desktop);
+static void atk_component_interface_init (AtkComponentIface *iface);
+static void spi_atk_desktop_get_extents	 (AtkComponent    *component,
+                                          gint            *x,
+                                          gint            *y,
+                                          gint            *width,
+                                          gint            *height,
+                                          AtkCoordType    coord_type);
+
+static GType 
+spi_atk_desktop_get_type (void)
+{
+  static GType type = 0;
+
+  if (!type)
+    {
+      static const GTypeInfo typeInfo =
+      {
+  	sizeof (SpiAtkDesktopClass),
+  	(GBaseInitFunc) NULL,
+  	(GBaseFinalizeFunc) NULL,
+  	(GClassInitFunc) NULL,
+  	(GClassFinalizeFunc) NULL,
+  	NULL,
+  	sizeof (SpiAtkDesktop),
+  	0,
+  	(GInstanceInitFunc) spi_atk_desktop_init,
+      } ;
+      static const GInterfaceInfo atk_component_info =
+	{
+        (GInterfaceInitFunc) atk_component_interface_init,
+	(GInterfaceFinalizeFunc) NULL,
+	NULL
+      };
+
+      type = g_type_register_static (ATK_TYPE_OBJECT,
+                                     "SpiAtkDesktop", &typeInfo, 0);
+      g_type_add_interface_static (type, ATK_TYPE_COMPONENT,
+                                   &atk_component_info);
+    }
+  return type;
+}
+
+static void 
+spi_atk_desktop_init (SpiAtkDesktop *desktop)
+{
+  GdkDisplay *display;
+
+  atk_object_set_name (ATK_OBJECT (desktop), "main");
+  display = gdk_x11_lookup_xdisplay (GDK_DISPLAY ());
+  desktop->screen = gdk_display_get_default_screen (display);
+}
+
+static void
+atk_component_interface_init (AtkComponentIface *iface)
+{
+  g_return_if_fail (iface != NULL);
+
+  iface->get_extents = spi_atk_desktop_get_extents;
+}
+
+static void 
+spi_atk_desktop_get_extents (AtkComponent *component,
+                             gint         *x,
+                             gint	  *y,
+                             gint	  *width,
+                             gint	  *height,
+                             AtkCoordType coord_type)
+{
+  SpiAtkDesktop *desktop;
+
+  g_return_if_fail (SPI_IS_ATK_DESKTOP (component));
+  desktop = SPI_ATK_DESKTOP (component);
+  *x = 0;
+  *y = 0;
+  *width = gdk_screen_get_width (desktop->screen);
+  *height = gdk_screen_get_height (desktop->screen);
+}
+
 static void
 spi_desktop_init (SpiDesktop *desktop)
 {
-  spi_base_construct (SPI_BASE (desktop), g_object_new (ATK_TYPE_OBJECT, NULL));
-
   desktop->applications = NULL;
   bonobo_object_set_immortal (BONOBO_OBJECT (desktop), TRUE);
-
-  atk_object_set_name (ATK_OBJECT (SPI_BASE (desktop)->gobj), "main");
 }
 
 static void
@@ -116,6 +211,12 @@ impl_desktop_get_child_at_index (PortableServer_Servant servant,
   return (Accessibility_Accessible) retval;
 }
 
+static void
+spi_desktop_exiting (void)
+{
+  exiting = TRUE;
+}
+
 static void
 spi_desktop_class_init (SpiDesktopClass *klass)
 {
@@ -147,19 +248,25 @@ spi_desktop_class_init (SpiDesktopClass *klass)
 		  1, G_TYPE_UINT);
   epv->_get_childCount = impl_desktop_get_child_count;
   epv->getChildAtIndex = impl_desktop_get_child_at_index;
+  g_atexit (spi_desktop_exiting);
 }
 
 BONOBO_TYPE_FUNC_FULL (SpiDesktop,
 		       Accessibility_Desktop,
 		       PARENT_TYPE,
-		       spi_desktop);
+		       spi_desktop)
 
 SpiDesktop *
 spi_desktop_new (void)
 {
-  SpiDesktop *retval = g_object_new (SPI_DESKTOP_TYPE, NULL);
+  SpiDesktop *desktop; 
+  SpiAccessible *accessible; 
 
-  return retval;
+  accessible = spi_accessible_construct (SPI_DESKTOP_TYPE, g_object_new (SPI_TYPE_ATK_DESKTOP, NULL));
+  g_assert (SPI_IS_DESKTOP (accessible));
+  desktop = SPI_DESKTOP (accessible);
+
+  return desktop;
 }
 
 static void
@@ -167,7 +274,8 @@ abnormal_application_termination (gpointer object, Application *app)
 {
   g_return_if_fail (SPI_IS_DESKTOP (app->desktop));
 
-  spi_desktop_remove_application (app->desktop, app->ref);
+  if (!exiting)
+    spi_desktop_remove_application (app->desktop, app->ref);
 }
 
 void
